#ifndef __EDG_WORKLOAD_LOGGING_CLIENT_LOGGING_EXCEPTIONS_HPP__
#define __EDG_WORKLOAD_LOGGING_CLIENT_LOGGING_EXCEPTIONS_HPP__

#ident "$Header: /cvs/glite/org.glite.lb.client-interface/interface/Attic/LoggingExceptions.h,v 1.6.22.1 2010/03/16 09:12:22 zsustr Exp $"
/*
Copyright (c) Members of the EGEE Collaboration. 2004-2010.
See http://www.eu-egee.org/partners for details on the copyright holders.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/


/** @file LoggingExceptions.h
 *  @version $Revision: 1.6.22.1 $
 */

#include "glite/wmsutils/exception/Exception.h"

#include <pthread.h>

EWL_BEGIN_NAMESPACE

/** Base class for all exceptions thrown by the L&B C++ classes.
 *
 * This class serves as a common base for all exceptions thrown by the
 * L&B C++ API classes. In case when the exception is constructed from
 * another exception (creating chained exception list), the error
 * message is created by concatenating the error message of the
 * original exception and the new error message. All the other
 * functionality (printing error message, logging it, printing stack
 * trace) is inherited from the base class glite::wmsutils::exception::Exception.
 */
class Exception: public glite::wmsutils::exception::Exception {
public:
	
	/** Constructor for mandatory fields.
	 *
	 * Updates all the mandatory fields and names the exception.
	 * \param[in] source 		Source filename where the exception was raised.
	 * \param[in] line_number 	Line in the source that caused the exception.
	 * \param[in] method 		Name of the method that raised the exception.
	 * \param[in] code 		Error code giving the reason for exception.
	 * \param[in] exception 	Error message describing the exception.
	 */
	Exception(const std::string& source,
		  int line_number,
		  const std::string& method,
		  int   code,
		  const std::string& exception) 
		: glite::wmsutils::exception::Exception(source, 
							line_number, 
							method, 
							code, 
							"glite::lb::Exception")
		{ error_message = exception; };
	
	/** Constructor for mandatory fields and the exception chain.
	 *
	 * Updates all the mandatory fields, names the exception and
	 * adds the original exception's error message to the current
	 * one.
	 * \param[in] source 		Source filename where the exception was raised.
	 * \param[in] line_number 	Line in the source that caused the exception.
	 * \param[in] method 		Name of the method that raised the exception.
	 * \param[in] code 		Error code giving the reason for exception.
	 * \param[in] exception 	Error message describing the exception.
	 * \param[in] exc 		Originally raised exception.
	 */
	Exception(const std::string& source,
		  int line_number,
		  const std::string& method,
		  int   code,
		  const std::string& exception,
		  const glite::wmsutils::exception::Exception &exc)
		: glite::wmsutils::exception::Exception(source, 
							line_number, 
							method, 
							code, 
							"glite::lb::Exception")
		{ error_message = exception + ": " + exc.what(); };
};


/** Exception encapsulating error states originating in the L&B.
 *
 * This class is simple child of the base Exception class, adding no
 * new functionality. Its purpose is to differentiate the error
 * conditions originating in the L&B subsystem from other errors (such
 * as system ones).
 */
class LoggingException: public Exception {
public:
	
	/** Constructor for mandatory fields.
	 *
	 * Updates all the mandatory fields and names the exception.
	 * \param[in] source 		Source filename where the exception was raised.
	 * \param[in] line_number 	Line in the source that caused the exception.
	 * \param[in] method 		Name of the method that raised the exception.
	 * \param[in] code 		Error code giving the reason for exception.
	 * \param[in] exception 	Error message describing the exception.
	 */
	LoggingException(const std::string& source,
			 int line_number,
			 const std::string& method,
			 int   code,
			 const std::string& exception) 
		: Exception(source, line_number, method, code, exception)
		{};
	
	/** Constructor for mandatory fields and the exception chain.
	 *
	 * Updates all the mandatory fields, names the exception and
	 * adds the original exception's error message to the current
	 * one.
	 * \param[in] source 		Source filename where the exception was raised.
	 * \param[in] line_number 	Line in the source that caused the exception.
	 * \param[in] method 		Name of the method that raised the exception.
	 * \param[in] code 		Error code giving the reason for exception.
	 * \param[in] exception 	Error message describing the exception.
	 * \param[in] exc 		Originally raised exception.
	 */
	LoggingException(const std::string& source,
			 int line_number,
			 const std::string& method,
			 int   code,
			 const std::string& exception, 
			 const glite::wmsutils::exception::Exception &exc)
		: Exception(source, line_number, method, code, exception)
		{};
};


/** Exceptions caused by system errors.
 *
 * This class represents error conditions caused by failing system
 * calls. The error message is augmented with the system error message
 * obtained by calling strerror().
 */
class OSException: public Exception {
public:
	
	/** Constructor for mandatory fields.
	 *
	 * Updates all the mandatory fields and names the exception.
	 * \param[in] source 		Source filename where the exception was raised.
	 * \param[in] line_number 	Line in the source that caused the exception.
	 * \param[in] method 		Name of the method that raised the exception.
	 * \param[in] code 		Error code giving the reason for exception.
	 * \param[in] exception 	Error message describing the exception.
	 */
	OSException(const std::string& source,
		    int line_number,
		    const std::string& method,
		    int   code,
		    const std::string& exception)
		: Exception(source, 
			    line_number, 
			    method, 
			    code, 
			    exception + ": " + strerror(code))
		{};
	
	/** Constructor for mandatory fields and the exception chain.
	 *
	 * Updates all the mandatory fields, names the exception and
	 * adds the original exception's error message to the current
	 * one.
	 * \param[in] source 		Source filename where the exception was raised.
	 * \param[in] line_number 	Line in the source that caused the exception.
	 * \param[in] method 		Name of the method that raised the exception.
	 * \param[in] code 		Error code giving the reason for exception.
	 * \param[in] exception 	Error message describing the exception.
	 * \param[in] exc 		Originally raised exception.
	 */
	OSException(const std::string& source,
		    int line_number,
		    const std::string& method,
		    int   code,
		    const std::string& exception,
		    const glite::wmsutils::exception::Exception &exc)
		: Exception(source, 
			    line_number, 
			    method, 
			    code, 
			    exception + ": " + strerror(code))
		{};
};


/** Mandatory exception fields.
 *
 * This defines the mandatory parameters for all exception
 * constructors (filename, line, method name).
 */
#define EXCEPTION_MANDATORY                           \
	__FILE__,                                     \
        __LINE__,                                     \
        std::string(CLASS_PREFIX) + __FUNCTION__         

/** Stacking exceptions.
 *
 * This was originally used for creating the exception chain; now the
 * same result is achieved by adding the nested exception to the
 * constructor parameter list.
 */
#define STACK_ADD                                     

/** Utility macro to throw LoggingException.
 *
 * This macro is used to obtain the L&B error message and throw the
 * appropriate exception.
 * Note: we can use __LINE__ several times in macro, it is expanded into
 * one row.
 */
#define throw_exception(context, exception)           \
{ STACK_ADD;                                          \
  {                                                   \
     char *text, *desc;                               \
     int  code;                                       \
     std::string exc;                                      \
                                                      \
     code = edg_wll_Error((context), &text, &desc);   \
     exc = exception;                                 \
     if (text) {				      \
     	exc += ": ";                                  \
    	exc += text;                                  \
     }						      \
     if (desc) {				      \
	exc += ": ";                                  \
	exc += desc;                                  \
     }						      \
     free(text);                                      \
     free(desc);                                      \
     throw LoggingException(EXCEPTION_MANDATORY,      \
	                    code,                     \
		            exc);                     \
  }						      \
}
 
/** Utility macro to check result of L&B calls.
 *
 * Checks return value of L&B calls and throws exception if the code
 * failed.
 */
#define check_result(code, context, desc)             \
  if((code)) throw_exception((context), desc)



EWL_END_NAMESPACE

#endif
